"use strict";
exports.__esModule = true;
var murmurhash_1 = require("murmurhash");
var EMPTY_OBJECT = '__empty__obj';
var EMPTY_ARRAY = '__empty__arr';
var Node = /** @class */ (function () {
    function Node() {
        this.children = [];
        this.hash = 0;
    }
    return Node;
}());
var hasher = function (thing, prefix) {
    if (prefix === void 0) { prefix = ''; }
    var stringThing = prefix + (typeof thing) + '::' + thing;
    return murmurhash_1.v3(stringThing);
};
var combineHashes = function (parentHash, child) {
    /* tslint:disable:no-bitwise */
    return (parentHash + child.hash) & 0xFFFFFFFF;
};
var newNode = function (thing, prefix) {
    var node = new Node();
    node.hash = hasher(thing, prefix);
    return node;
};
var createTree = function (currNode, currentInput, prefix) {
    if (prefix === void 0) { prefix = ''; }
    var isObject = typeof currentInput === 'object';
    var isNil = currentInput === null || typeof currentInput === 'undefined';
    var isArray = Array.isArray(currentInput);
    var keys = !isNil ? Object.keys(currentInput) : [];
    // we're at a weird value
    if (currentInput instanceof Date || currentInput instanceof RegExp) {
        return newNode(currentInput, prefix);
    }
    // if we're at a value
    if (!isObject && !isArray) {
        return newNode(currentInput, prefix);
    }
    // if we're at an iterable
    if (!keys.length) {
        return isArray ? newNode(EMPTY_ARRAY, prefix) : newNode(EMPTY_OBJECT, prefix);
    }
    for (var _i = 0, keys_1 = keys; _i < keys_1.length; _i++) {
        var key = keys_1[_i];
        var pfx = void 0;
        if (!isArray && isObject) { // if we're dealing with an object prefix the key
            pfx = key;
        }
        var node = createTree(new Node(), currentInput[key], pfx);
        currNode.children.push(node);
    }
    // iterable's hash is combined hash of all children
    var combined = currNode.children.reduce(combineHashes, 0);
    currNode.hash = hasher(combined);
    return currNode;
};
var createFinalHash = function (input) {
    var tree = createTree(new Node(), input);
    return tree.hash;
};
exports.compare = function (a, b) {
    if (a && b && ((Array.isArray(a) && Array.isArray(b)) || (typeof a === 'object' && typeof b === 'object')) && (a.length === b.length)) {
        return createFinalHash(a) === createFinalHash(b);
    }
    return false;
};
